@@ -1,23 +1,3 @@
-## ----------------------------------------------------------------------------|
-# For discrete data: to transfer raw data for the algorithm 
-## ----------------------------------------------------------------------------|
-data_transform = function( raw_data )
-{
-	all_patterns    = apply( raw_data, 1, function( x ){ paste( x, collapse = '' ) } )   
-	unique_patterns = unique( all_patterns )
-	   
-	length_unique_patterns = length( unique_patterns )
-	data  = matrix( 0, nrow = length_unique_patterns, ncol = ncol( raw_data ) + 1 )
-	   
-	for( i in seq_len( length_unique_patterns ) )
-	{
-		which_one = which( all_patterns == unique_patterns[i] )
-		data[i, ] = c( raw_data[ which_one[1], ], length( which_one ) )
-	}
-	 
-	return( data )
-}
-   
 ## ----------------------------------------------------------------------------|
 # Main function of BDgraph package: BDMCMC algorithm for graphical models 
 ## ----------------------------------------------------------------------------|
@@ -33,7 +13,7 @@ bdgraph.mpl = function( data, n = NULL, method = "ggm", transfer = TRUE, algorit
 
 	if( !is.matrix( data ) & !is.data.frame( data ) ) stop( "Data should be a matrix or dataframe" )
 	if( is.data.frame( data ) ) data <- data.matrix( data )
-	if( iter <= burnin )   stop( "Number of iteration must be more than number of burn-in" )
+	if( iter < burnin )   stop( "Number of iteration must be more than number of burn-in" )
 
 	if( any( is.na( data ) ) ) stop( "This method does not deal with missing value. You could choose method = gcgm" )	
 		
@@ -91,7 +71,7 @@ bdgraph.mpl = function( data, n = NULL, method = "ggm", transfer = TRUE, algorit
 		qp1           = ( p * ( p - 1 ) / 2 ) + 1
 		string_g      = paste( c( rep( 0, qp1 ) ), collapse = '' )
 		sample_graphs = c( rep ( string_g, iter - burnin ) )  # vector of numbers like "10100" 
-		graph_weights = c( rep ( 0, iter - burnin ) )         # waiting time for every state
+		graph_weights = c( rep ( 1, iter - burnin ) )         # waiting time for every state
 		all_graphs    = c( rep ( 0, iter - burnin ) )         # vector of numbers like "10100"
 		all_weights   = c( rep ( 1, iter - burnin ) )         # waiting time for every state		
 		size_sample_g = 0
@@ -121,10 +101,18 @@ bdgraph.mpl = function( data, n = NULL, method = "ggm", transfer = TRUE, algorit
 		mes <- paste( c( iter, " iteration is started.                    " ), collapse = "" )
 		cat( mes, "\r" )
 	}
-
+   
 ## ----------------------------------------------------------------------------|
 	if( save.all == TRUE )
 	{
+		if( ( method == "ggm" ) && ( algorithm == "rjmcmc" ) )
+		{
+			result = .C( "ggm_rjmcmc_mpl_map", as.integer(iter), as.integer(burnin), G = as.integer(G), as.double(S), as.integer(n), as.integer(p),
+						all_graphs = as.integer(all_graphs), all_weights = as.double(all_weights), 
+						sample_graphs = as.character(sample_graphs), graph_weights = as.double(graph_weights), size_sample_g = as.integer(size_sample_g),
+						as.integer(print), PACKAGE = "BDgraph" )
+		}
+    
 		if( ( method == "ggm" ) && ( algorithm == "bdmcmc" ) && ( multi_update == 1 ) )
 		{
 			result = .C( "ggm_bdmcmc_mpl_map", as.integer(iter), as.integer(burnin), G = as.integer(G), as.double(S), as.integer(n), as.integer(p),
@@ -132,7 +120,7 @@ bdgraph.mpl = function( data, n = NULL, method = "ggm", transfer = TRUE, algorit
 						sample_graphs = as.character(sample_graphs), graph_weights = as.double(graph_weights), size_sample_g = as.integer(size_sample_g),
 						as.integer(print), PACKAGE = "BDgraph" )
 		}
-
+    
 		if( ( method == "ggm" ) && ( algorithm == "bdmcmc" ) && ( multi_update != 1 ) )
 		{
 			counter_all_g = 0
@@ -141,7 +129,16 @@ bdgraph.mpl = function( data, n = NULL, method = "ggm", transfer = TRUE, algorit
 						sample_graphs = as.character(sample_graphs), graph_weights = as.double(graph_weights), size_sample_g = as.integer(size_sample_g), counter_all_g = as.integer(counter_all_g),
 						as.integer(multi_update), as.integer(print), PACKAGE = "BDgraph" )
 		}
-      
+
+		if( ( method == "dgm" ) && ( algorithm == "rjmcmc" ) )
+		{
+			result = .C( "dgm_rjmcmc_mpl_map", as.integer(iter), as.integer(burnin), G = as.integer(G), 
+			            as.integer(data), as.integer(freq_data), as.integer(length_f_data), as.integer(max_range_nodes), as.double(alpha), as.integer(n), as.integer(p),
+						all_graphs = as.integer(all_graphs), all_weights = as.double(all_weights), 
+						sample_graphs = as.character(sample_graphs), graph_weights = as.double(graph_weights), size_sample_g = as.integer(size_sample_g),
+						as.integer(print), PACKAGE = "BDgraph" )
+		}
+
 		if( ( method == "dgm" ) && ( algorithm == "bdmcmc" ) && ( multi_update == 1 ) )
 		{
 			result = .C( "dgm_bdmcmc_mpl_map", as.integer(iter), as.integer(burnin), G = as.integer(G), 
@@ -163,7 +160,13 @@ bdgraph.mpl = function( data, n = NULL, method = "ggm", transfer = TRUE, algorit
       
 	}else{
 		
-		if( ( method == "ggm" ) && ( algorithm == "bdmcmc" ) && ( multi_update == 1 )  )
+		if( ( method == "ggm" ) && ( algorithm == "rjmcmc" ) )
+		{
+			result = .C( "ggm_rjmcmc_mpl_ma", as.integer(iter), as.integer(burnin), G = as.integer(G), as.double(S), as.integer(n), as.integer(p), 
+						 p_links = as.double(p_links), as.integer(print), PACKAGE = "BDgraph" )
+		}
+		
+		if( ( method == "ggm" ) && ( algorithm == "bdmcmc" ) && ( multi_update == 1 ) )
 		{
 			result = .C( "ggm_bdmcmc_mpl_ma", as.integer(iter), as.integer(burnin), G = as.integer(G), as.double(S), as.integer(n), as.integer(p), 
 						 p_links = as.double(p_links), as.integer(print), PACKAGE = "BDgraph" )
@@ -175,7 +178,14 @@ bdgraph.mpl = function( data, n = NULL, method = "ggm", transfer = TRUE, algorit
 						p_links = as.double(p_links), as.integer(multi_update), as.integer(print), PACKAGE = "BDgraph" )
 		}				
 
-		if( ( method == "dgm" ) && ( algorithm == "bdmcmc" ) && ( multi_update == 1 )  )
+		if( ( method == "dgm" ) && ( algorithm == "rjmcmc" ) )
+		{
+			result = .C( "dgm_rjmcmc_mpl_ma", as.integer(iter), as.integer(burnin), G = as.integer(G), 
+			            as.integer(data), as.integer(freq_data), as.integer(length_f_data), as.integer(max_range_nodes), as.double(alpha), 
+						as.integer(n), as.integer(p), p_links = as.double(p_links), as.integer(print), PACKAGE = "BDgraph" )
+		}
+
+		if( ( method == "dgm" ) && ( algorithm == "bdmcmc" ) && ( multi_update == 1 ) )
 		{
 			result = .C( "dgm_bdmcmc_mpl_ma", as.integer(iter), as.integer(burnin), G = as.integer(G), 
 			            as.integer(data), as.integer(freq_data), as.integer(length_f_data), as.integer(max_range_nodes), as.double(alpha), 
@@ -193,8 +203,7 @@ bdgraph.mpl = function( data, n = NULL, method = "ggm", transfer = TRUE, algorit
 
 	if( algorithm != "hc" )
 	{
-		last_graph = matrix( result $ G, p, p )
-
+		last_graph             = matrix( result $ G, p, p )
 		colnames( last_graph ) = colnames_data[1:p]
 
 		if( save.all == TRUE )
@@ -204,7 +213,7 @@ bdgraph.mpl = function( data, n = NULL, method = "ggm", transfer = TRUE, algorit
 			graph_weights = result $ graph_weights[ 1 : size_sample_g ]
 			all_graphs    = result $ all_graphs + 1
 			all_weights   = result $ all_weights	
-			if( multi_update != 1 )
+			if( ( algorithm != "rjmcmc" ) & ( multi_update != 1 ) )
 			{ 
 				all_weights = all_weights[ 1 : ( result $ counter_all_g ) ]
 				all_graphs  = all_graphs[  1 : ( result $ counter_all_g ) ] 
@@ -214,6 +223,7 @@ bdgraph.mpl = function( data, n = NULL, method = "ggm", transfer = TRUE, algorit
 						all_graphs = all_graphs, all_weights = all_weights, last_graph = last_graph )
 		}else{
 			p_links   = matrix( result $ p_links, p, p ) 
+			if( algorithm == "rjmcmc" )	p_links = p_links / ( iter - burnin )
 			p_links[ lower.tri( p_links ) ] = 0
 			colnames( p_links ) = colnames_data[1:p]
 			output = list( p_links = p_links, last_graph = last_graph )
@@ -230,4 +240,24 @@ bdgraph.mpl = function( data, n = NULL, method = "ggm", transfer = TRUE, algorit
 	return( output )   
 }
          
+## ----------------------------------------------------------------------------|
+# For discrete data: to transfer raw data for the algorithm 
+## ----------------------------------------------------------------------------|
+data_transform = function( raw_data )
+{
+	all_patterns    = apply( raw_data, 1, function( x ){ paste( x, collapse = '' ) } )   
+	unique_patterns = unique( all_patterns )
+	   
+	length_unique_patterns = length( unique_patterns )
+	data  = matrix( 0, nrow = length_unique_patterns, ncol = ncol( raw_data ) + 1 )
+	   
+	for( i in seq_len( length_unique_patterns ) )
+	{
+		which_one = which( all_patterns == unique_patterns[i] )
+		data[i, ] = c( raw_data[ which_one[1], ], length( which_one ) )
+	}
+	 
+	return( data )
+}
+   
 
