@@ -39,7 +39,7 @@ void ggm_DMH_bdmcmc_ma( int *iter, int *burnin, int G[], double Ts[], double Ti[
 	int i, j, ij, jj, counter;
 	int dim = *p, pxp = dim * dim, p1 = dim - 1, p1xp1 = p1 * p1, p2 = dim - 2, p2xp2 = p2 * p2, p2x2 = p2 * 2, one = 1;
 
-	double Dsijj, Dsjj, Dsij, sum_weights = 0.0, K022, weight_C;
+	double Dsijj, Dsjj, Dsij, sum_weights = 0.0, weight_C;
 	double log_rate, sum_rates;
 	
 	vector<double> p_links_Cpp( pxp, 0.0 ); 
@@ -114,8 +114,8 @@ void ggm_DMH_bdmcmc_ma( int *iter, int *burnin, int G[], double Ts[], double Ti[
 		// STEP 1: calculating birth and death rates --------------------------|		
 		for( j = 1; j < dim; j++ )
 		{			
-			jj    = j * dim + j;
-			Dsjj  = Ds[jj];
+			jj   = j * dim + j;
+			Dsjj = Ds[jj];
 			Djj  = D[jj];
 
 			for( i = 0; i < j; i++ )
@@ -127,13 +127,13 @@ void ggm_DMH_bdmcmc_ma( int *iter, int *burnin, int G[], double Ts[], double Ti[
 				Dijj  = - Dij * Dij / Djj;
 
 				log_H_ij( &K[0], &sigma[0], &logH_ij, &i, &j,
-					   &Kj22_inv[0], &Kj12[0], &Kj12xK22_inv[0], &K022, &K12[0], &K22_inv[0], &K12xK22_inv[0], &K121[0], 
+					   &Kj22_inv[0], &Kj12[0], &Kj12xK22_inv[0], &K12[0], &K22_inv[0], &K12xK22_inv[0], &K121[0], 
 					   &sigmaj12[0], &sigmaj22[0], &sigma11[0], &sigma12[0], &sigma22[0], &sigma11_inv[0], &sigma21xsigma11_inv[0], &sigma2112[0],
 					   &dim, &p1, &p2, &p2xp2, &jj,
 					   &Dsijj, &Dsij, &Dsjj );
 
 				log_H_ij( &K_dmh[0], &sigma_dmh[0], &logI_p, &i, &j,
-					   &Kj22_inv[0], &Kj12[0], &Kj12xK22_inv[0], &K022, &K12[0], &K22_inv[0], &K12xK22_inv[0], &K121[0], 
+					   &Kj22_inv[0], &Kj12[0], &Kj12xK22_inv[0], &K12[0], &K22_inv[0], &K12xK22_inv[0], &K121[0], 
 					   &sigmaj12[0], &sigmaj22[0], &sigma11[0], &sigma12[0], &sigma22[0], &sigma11_inv[0], &sigma21xsigma11_inv[0], &sigma2112[0],
 					   &dim, &p1, &p2, &p2xp2, &jj,
 					   &Dijj, &Dij, &Djj );
@@ -213,7 +213,7 @@ void ggm_DMH_bdmcmc_map( int *iter, int *burnin, int G[], double Ts[], double Ti
 	int i, j, ij, jj, counter;
 	int dim = *p, pxp = dim * dim, p1 = dim - 1, p1xp1 = p1 * p1, p2 = dim - 2, p2xp2 = p2 * p2, p2x2 = p2 * 2, one = 1;
 
-	double Dsijj, Dsjj, Dsij, sum_weights = 0.0, K022, weight_C;
+	double Dsijj, Dsjj, Dsij, sum_weights = 0.0, weight_C;
 	double log_rate, sum_rates;
 	
 	vector<double> sigma( pxp ); 
@@ -301,13 +301,13 @@ void ggm_DMH_bdmcmc_map( int *iter, int *burnin, int G[], double Ts[], double Ti
 				Dijj  = - Dij * Dij / Djj;
 
 				log_H_ij( &K[0], &sigma[0], &logH_ij, &i, &j,
-					   &Kj22_inv[0], &Kj12[0], &Kj12xK22_inv[0], &K022, &K12[0], &K22_inv[0], &K12xK22_inv[0], &K121[0], 
+					   &Kj22_inv[0], &Kj12[0], &Kj12xK22_inv[0], &K12[0], &K22_inv[0], &K12xK22_inv[0], &K121[0], 
 					   &sigmaj12[0], &sigmaj22[0], &sigma11[0], &sigma12[0], &sigma22[0], &sigma11_inv[0], &sigma21xsigma11_inv[0], &sigma2112[0],
 					   &dim, &p1, &p2, &p2xp2, &jj,
 					   &Dsijj, &Dsij, &Dsjj );
 
 				log_H_ij( &K_dmh[0], &sigma_dmh[0], &logI_p, &i, &j,
-					   &Kj22_inv[0], &Kj12[0], &Kj12xK22_inv[0], &K022, &K12[0], &K22_inv[0], &K12xK22_inv[0], &K121[0], 
+					   &Kj22_inv[0], &Kj12[0], &Kj12xK22_inv[0], &K12[0], &K22_inv[0], &K12xK22_inv[0], &K121[0], 
 					   &sigmaj12[0], &sigmaj22[0], &sigma11[0], &sigma12[0], &sigma22[0], &sigma11_inv[0], &sigma21xsigma11_inv[0], &sigma2112[0],
 					   &dim, &p1, &p2, &p2xp2, &jj,
 					   &Dijj, &Dij, &Djj );
@@ -409,7 +409,7 @@ void ggm_DMH_bdmcmc_ma_multi_update( int *iter, int *burnin, int G[], double Ts[
 	int i, j, ij, jj, counter;
 	int dim = *p, pxp = dim * dim, p1 = dim - 1, p1xp1 = p1 * p1, p2 = dim - 2, p2xp2 = p2 * p2, p2x2 = p2 * 2, one = 1;
 
-	double Dsijj, Dsjj, Dsij, sum_weights = 0.0, K022, weight_C;
+	double Dsijj, Dsjj, Dsij, sum_weights = 0.0, weight_C;
 	double log_rate, sum_rates;
 
 	vector<double> p_links_Cpp( pxp, 0.0 ); 
@@ -502,13 +502,13 @@ void ggm_DMH_bdmcmc_ma_multi_update( int *iter, int *burnin, int G[], double Ts[
 				Dijj  = - Dij * Dij / Djj;
 
 				log_H_ij( &K[0], &sigma[0], &logH_ij, &i, &j,
-					   &Kj22_inv[0], &Kj12[0], &Kj12xK22_inv[0], &K022, &K12[0], &K22_inv[0], &K12xK22_inv[0], &K121[0], 
+					   &Kj22_inv[0], &Kj12[0], &Kj12xK22_inv[0], &K12[0], &K22_inv[0], &K12xK22_inv[0], &K121[0], 
 					   &sigmaj12[0], &sigmaj22[0], &sigma11[0], &sigma12[0], &sigma22[0], &sigma11_inv[0], &sigma21xsigma11_inv[0], &sigma2112[0],
 					   &dim, &p1, &p2, &p2xp2, &jj,
 					   &Dsijj, &Dsij, &Dsjj );
 
 				log_H_ij( &K_dmh[0], &sigma_dmh[0], &logI_p, &i, &j,
-					   &Kj22_inv[0], &Kj12[0], &Kj12xK22_inv[0], &K022, &K12[0], &K22_inv[0], &K12xK22_inv[0], &K121[0], 
+					   &Kj22_inv[0], &Kj12[0], &Kj12xK22_inv[0], &K12[0], &K22_inv[0], &K12xK22_inv[0], &K121[0], 
 					   &sigmaj12[0], &sigmaj22[0], &sigma11[0], &sigma12[0], &sigma22[0], &sigma11_inv[0], &sigma21xsigma11_inv[0], &sigma2112[0],
 					   &dim, &p1, &p2, &p2xp2, &jj,
 					   &Dijj, &Dij, &Djj );
@@ -593,7 +593,7 @@ void ggm_DMH_bdmcmc_map_multi_update( int *iter, int *burnin, int G[], double Ts
 	int i, j, ij, jj, counter;
 	int dim = *p, pxp = dim * dim, p1 = dim - 1, p1xp1 = p1 * p1, p2 = dim - 2, p2xp2 = p2 * p2, p2x2 = p2 * 2, one = 1;
 
-	double Dsijj, Dsjj, Dsij, sum_weights = 0.0, K022, weight_C;
+	double Dsijj, Dsjj, Dsij, sum_weights = 0.0, weight_C;
 	double log_rate, sum_rates;
 
 	vector<double> sigma( pxp ); 
@@ -684,13 +684,13 @@ void ggm_DMH_bdmcmc_map_multi_update( int *iter, int *burnin, int G[], double Ts
 				Dijj  = - Dij * Dij / Djj;
 
 				log_H_ij( &K[0], &sigma[0], &logH_ij, &i, &j,
-					   &Kj22_inv[0], &Kj12[0], &Kj12xK22_inv[0], &K022, &K12[0], &K22_inv[0], &K12xK22_inv[0], &K121[0], 
+					   &Kj22_inv[0], &Kj12[0], &Kj12xK22_inv[0], &K12[0], &K22_inv[0], &K12xK22_inv[0], &K121[0], 
 					   &sigmaj12[0], &sigmaj22[0], &sigma11[0], &sigma12[0], &sigma22[0], &sigma11_inv[0], &sigma21xsigma11_inv[0], &sigma2112[0],
 					   &dim, &p1, &p2, &p2xp2, &jj,
 					   &Dsijj, &Dsij, &Dsjj );
 
 				log_H_ij( &K_dmh[0], &sigma_dmh[0], &logI_p, &i, &j,
-					   &Kj22_inv[0], &Kj12[0], &Kj12xK22_inv[0], &K022, &K12[0], &K22_inv[0], &K12xK22_inv[0], &K121[0], 
+					   &Kj22_inv[0], &Kj12[0], &Kj12xK22_inv[0], &K12[0], &K22_inv[0], &K12xK22_inv[0], &K121[0], 
 					   &sigmaj12[0], &sigmaj22[0], &sigma11[0], &sigma12[0], &sigma22[0], &sigma11_inv[0], &sigma21xsigma11_inv[0], &sigma2112[0],
 					   &dim, &p1, &p2, &p2xp2, &jj,
 					   &Dijj, &Dij, &Djj );
@@ -795,7 +795,7 @@ void ggm_DMH_rjmcmc_ma( int *iter, int *burnin, int G[], double Ts[], double Ti[
 	int i, j, ij, jj, counter;
 	int dim = *p, pxp = dim * dim, p1 = dim - 1, p1xp1 = p1 * p1, p2 = dim - 2, p2xp2 = p2 * p2, p2x2 = p2 * 2;
 
-	double Dsijj, Dsjj, Dsij, K022;
+	double Dsijj, Dsjj, Dsij;
 
 	vector<double> sigma( pxp ); 
 	vector<double> copyK( pxp ); 
@@ -879,13 +879,13 @@ void ggm_DMH_rjmcmc_ma( int *iter, int *burnin, int G[], double Ts[], double Ti[
 		rgwish_sigma( G, &size_node[0], Ti, &K_dmh[0], &sigma_dmh[0], &b1, &dim, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
 
 		log_H_ij( &K[0], &sigma[0], &logH_ij, &selected_edge_i, &selected_edge_j,
-               &Kj22_inv[0], &Kj12[0], &Kj12xK22_inv[0], &K022, &K12[0], &K22_inv[0], &K12xK22_inv[0], &K121[0], 
+               &Kj22_inv[0], &Kj12[0], &Kj12xK22_inv[0], &K12[0], &K22_inv[0], &K12xK22_inv[0], &K121[0], 
                &sigmaj12[0], &sigmaj22[0], &sigma11[0], &sigma12[0], &sigma22[0], &sigma11_inv[0], &sigma21xsigma11_inv[0], &sigma2112[0],
                &dim, &p1, &p2, &p2xp2, &jj,
                &Dsijj, &Dsij, &Dsjj );
 
 		log_H_ij( &K_dmh[0], &sigma_dmh[0], &logI_p, &selected_edge_i, &selected_edge_j,
-               &Kj22_inv[0], &Kj12[0], &Kj12xK22_inv[0], &K022, &K12[0], &K22_inv[0], &K12xK22_inv[0], &K121[0], 
+               &Kj22_inv[0], &Kj12[0], &Kj12xK22_inv[0], &K12[0], &K22_inv[0], &K12xK22_inv[0], &K121[0], 
                &sigmaj12[0], &sigmaj22[0], &sigma11[0], &sigma12[0], &sigma22[0], &sigma11_inv[0], &sigma21xsigma11_inv[0], &sigma2112[0],
                &dim, &p1, &p2, &p2xp2, &jj,
                &Dijj, &Dij, &Djj );
@@ -948,7 +948,7 @@ void ggm_DMH_rjmcmc_map( int *iter, int *burnin, int G[], double Ts[], double Ti
 	int i, j, ij, jj, counter;
 	int dim = *p, pxp = dim * dim, p1 = dim - 1, p1xp1 = p1 * p1, p2 = dim - 2, p2xp2 = p2 * p2, p2x2 = p2 * 2;
 
-	double Dsijj, Dsjj, Dsij, K022;
+	double Dsijj, Dsjj, Dsij;
 
 	vector<double> sigma( pxp ); 
 	vector<double> copyK( pxp ); 
@@ -1034,13 +1034,13 @@ void ggm_DMH_rjmcmc_map( int *iter, int *burnin, int G[], double Ts[], double Ti
 		rgwish_sigma( G, &size_node[0], Ti, &K_dmh[0], &sigma_dmh[0], &b1, &dim, &sigma_start[0], &inv_C[0], &beta_star[0], &sigma_i[0], sigma_start_N_i, sigma_N_i, N_i );		
 
 		log_H_ij( &K[0], &sigma[0], &logH_ij, &selected_edge_i, &selected_edge_j,
-               &Kj22_inv[0], &Kj12[0], &Kj12xK22_inv[0], &K022, &K12[0], &K22_inv[0], &K12xK22_inv[0], &K121[0], 
+               &Kj22_inv[0], &Kj12[0], &Kj12xK22_inv[0], &K12[0], &K22_inv[0], &K12xK22_inv[0], &K121[0], 
                &sigmaj12[0], &sigmaj22[0], &sigma11[0], &sigma12[0], &sigma22[0], &sigma11_inv[0], &sigma21xsigma11_inv[0], &sigma2112[0],
                &dim, &p1, &p2, &p2xp2, &jj,
                &Dsijj, &Dsij, &Dsjj );
 
 		log_H_ij( &K_dmh[0], &sigma_dmh[0], &logI_p, &selected_edge_i, &selected_edge_j,
-               &Kj22_inv[0], &Kj12[0], &Kj12xK22_inv[0], &K022, &K12[0], &K22_inv[0], &K12xK22_inv[0], &K121[0], 
+               &Kj22_inv[0], &Kj12[0], &Kj12xK22_inv[0], &K12[0], &K22_inv[0], &K12xK22_inv[0], &K121[0], 
                &sigmaj12[0], &sigmaj22[0], &sigma11[0], &sigma12[0], &sigma22[0], &sigma11_inv[0], &sigma21xsigma11_inv[0], &sigma2112[0],
                &dim, &p1, &p2, &p2xp2, &jj,
                &Dijj, &Dij, &Djj );
