@@ -1,20 +1,32 @@
+// ----------------------------------------------------------------------------|
+//     Copyright (C) 2012-2016 Mohammadi A. and Wit C. E.
+//
+//     This file is part of BDgraph package.
+//
+//     BDgraph is free software: you can redistribute it and/or modify it under 
+//     the terms of the GNU General Public License as published by the Free 
+//     Software Foundation; see <https://cran.r-project.org/web/licenses/GPL-3>.
+//
+//     Maintainer:
+//     Abdolreza Mohammadi: a.mohammadi@rug.nl or a.mohammadi@uvt.nl
+// ----------------------------------------------------------------------------|
+  
 #include "rgwish.h"
 
 // sampling from Wishart distribution // Ts = chol( solve( Ds ) )
-void rwish( double Ts[], double K[], int *b, int *p )
+void rwish_c( double Ts[], double K[], int *b, int *p )
 {
+    GetRNGstate();
 	int i, j, dim = *p, pxp = dim * dim, bK = *b;
 	vector<double> psi( pxp, 0.0 ); 
 
 	// ---- Sample values in Psi matrix ---------------------------------------|
-    GetRNGstate();
 	for( i = 0; i < dim; i++ )
 		psi[i * dim + i] = sqrt( rchisq( bK + dim - i - 1 ) );
 
 	for( j = 1; j < dim; j++ )
 		for( i = 0; i < j; i++ )
 			psi[j * dim + i] = rnorm( 0, 1 );
-	PutRNGstate();
 	// ------------------------------------------------------------------------|
 
     	// C = psi %*% Ts   I used   psi = psi %*% Ts
@@ -26,18 +38,19 @@ void rwish( double Ts[], double K[], int *b, int *p )
 	// K = t(C) %*% C 
 	// LAPACK function to compute  C := alpha * A * B + beta * C																				
 	F77_NAME(dgemm)( &transT, &transN, &dim, &dim, &dim, &alpha, &psi[0], &dim, &psi[0], &dim, &beta, K, &dim );
+	PutRNGstate();
 }
 
 // G is adjacency matrix which has zero in its diagonal // threshold = 1e-8
-void rgwish( int G[], double Ts[], double K[], int *b, int *p, double *threshold )
+void rgwish_c( int G[], double Ts[], double K[], int *b, int *p )
 {
 	char transN  = 'N', uplo  = 'U'; 
 	double alpha = 1.0, beta  = 0.0;
 
 	int info, i, j, l, size_node, one = 1, dim = *p, pxp = dim * dim;	
-	double temp, threshold_C = *threshold;
+	double temp, threshold = 1e-8;
 	
-	rwish( Ts, K, b, &dim );
+	rwish_c( Ts, K, b, &dim );
 	
 	vector<double> sigma_start( pxp ); 
 	inverse( K, &sigma_start[0], &dim );
@@ -52,7 +65,7 @@ void rgwish( int G[], double Ts[], double K[], int *b, int *p, double *threshold
 	vector<double> sigma_N_i( pxp );         // For dynamic memory used
 
 	double max_diff = 1.0;	
-	while ( max_diff > threshold_C )
+	while ( max_diff > threshold )
 	{
 		memcpy( &sigma_last[0], &sigma[0], sizeof( double ) * pxp );
 		
@@ -126,17 +139,17 @@ void rgwish( int G[], double Ts[], double K[], int *b, int *p, double *threshold
 }
      
 // rgwish ONLY for inside of MCMC algorithm
-void rgwish_sigma( int G[], int size_node[], double Ts[], double K[], double sigma[], int *b_star, int *p, double *threshold,
+void rgwish_sigma( int G[], int size_node[], double Ts[], double K[], double sigma[], int *b_star, int *p,
 					double sigma_start[], double inv_C[], double beta_star[], double sigma_i[], 
 					vector<double> &sigma_start_N_i, vector<double> &sigma_N_i, vector<int> &N_i )
 {
 	int i, i1, j, ij, ip, l, size_node_i, info, one = 1, dim = *p, pxp = dim * dim, dim1 = dim + 1, bKdim = *b_star + dim - 1;	
-	double temp, threshold_C = *threshold, alpha = 1.0, beta  = 0.0;
+	double temp, threshold = 1e-8, alpha = 1.0, beta  = 0.0;
 	char transT  = 'T', transN = 'N', side = 'R', upper = 'U';																	
 	
 	// STEP 1: sampling from wishart distributions
 	// ---- Sample values in Psi matrix ---
-	GetRNGstate();
+	//GetRNGstate();
 	for( i = 0; i < dim; i++ )
 		sigma_start[i * dim1] = sqrt( rchisq( bKdim - i ) ); // i * dim1 = i * dim + i
 
@@ -146,7 +159,7 @@ void rgwish_sigma( int G[], int size_node[], double Ts[], double K[], double sig
 			sigma_start[j * dim + i] = rnorm( 0, 1 );
 			sigma_start[i * dim + j] = 0.0;
 		}
-	PutRNGstate();
+	//PutRNGstate();
 	// ------------------------------------
 	
 	// C = psi %*% Ts   I used psi = psi %*% Ts   Now is  sigma_start = sigma_start %*% Ts
@@ -166,7 +179,7 @@ void rgwish_sigma( int G[], int size_node[], double Ts[], double K[], double sig
 	memcpy( sigma, &sigma_start[0], sizeof( double ) * pxp ); 
 	
 	double max_diff = 1.0;	
-	while ( max_diff > threshold_C )
+	while ( max_diff > threshold )
 	{
 		max_diff = 0.0;
 		
@@ -261,16 +274,21 @@ void rgwish_sigma( int G[], int size_node[], double Ts[], double K[], double sig
 	F77_NAME(dposv)( &upper, &dim, &dim, &sigma_start[0], &dim, K, &dim, &info );
 }
 
-// Part of function for calculating I.g = function( G, b = 3, D = diag( ncol(G) ), mc = 100 ) out logIg
+// Part of function "gnorm"
+// which is for calculating Normalizing constant of G-Wishart distribution 
+// based on Monto Carlo algorithm
 void log_exp_mc( int G[], int nu[], int *b, double H[], int *check_H, int *mc, int *p, double f_T[] )
 {
+	GetRNGstate();
 	int iter, i, j, ij, h, r, mc_iter = *mc, dim = *p, pxp = dim * dim, b_c = *b;
 	double sumPsi, sumPsiH, sumPsiHi, sumPsiHj;
 	vector<double> psi( pxp, 0.0 );      
+
+	double max_numeric_limits_ld = numeric_limits<double>::max() / 1000;
+	double min_numeric_limits_ld = numeric_limits<double>::min() * 1000;
 	
-	GetRNGstate();
 	if( *check_H == 1 )
-	{
+	{ 
 		for( iter = 0; iter < mc_iter; iter++ ) 
 		{
 			for( i = 0; i < dim; i++ )
@@ -280,7 +298,10 @@ void log_exp_mc( int G[], int nu[], int *b, double H[], int *check_H, int *mc, i
 				for( j = i + 1; j < dim; j++ ) 
 				{
 					ij = j * dim + i;
-					if ( G[ij] == 1 ) psi[ij] = rnorm( 0, 1 );
+					if ( G[ij] == 1 ) 
+						psi[ij] = rnorm( 0, 1 );
+					else
+						psi[ij] = 0.0;
 				}
 			
 			for( i = 0; i < dim - 1; i++ ) 
@@ -290,23 +311,32 @@ void log_exp_mc( int G[], int nu[], int *b, double H[], int *check_H, int *mc, i
 					
 					if ( G[ij] == 0 )
 					{
-						psi[ij] = 0.0;
-
+						psi[ij] = 0.0; // it's not necessary
 						if ( i > 0 )  
 						{
 							sumPsi = 0.0;
 							//sum( psi[ 1 : ( i - 1 ), i ] * psi[ 1 : ( i - 1 ), j ] )
 							for( h = 0; h < ( i - 1 ); h++ )
-								sumPsi += ( psi[ i * dim + h ] * psi[ j * dim + h ] ); 
+							{
+								if( sumPsi == R_PosInf ) sumPsi = max_numeric_limits_ld;	
+								if( sumPsi == R_NegInf ) sumPsi = min_numeric_limits_ld;	
+								sumPsi += ( psi[ i * dim + h ] * psi[ j * dim + h ] );
+							}
 							
 							//psi[i, j] <- - sum( psi[ 1 : ( i - 1 ), i ] * psi[ 1 : ( i - 1 ), j ] ) / psi[i, i]
 							psi[ij] = - sumPsi / psi[ i * dim + i ];
 						}
 						
+						if( psi[ij] == R_PosInf ) psi[ij] = max_numeric_limits_ld;	
+						if( psi[ij] == R_NegInf ) psi[ij] = min_numeric_limits_ld;	
+												
 						//f_T[k] <- f_T[k] + psi[i, j] ^ 2
 						f_T[iter] += ( psi[ij] * psi[ij] ); 
 					}
 				}
+		
+			// checking Inf values
+			if( f_T[iter] == R_PosInf ) f_T[iter] = max_numeric_limits_ld;			
 		} 
 	}
 	else
@@ -320,7 +350,10 @@ void log_exp_mc( int G[], int nu[], int *b, double H[], int *check_H, int *mc, i
 				for( j = i + 1; j < dim; j++ ) 
 				{
 					ij = j * dim + i;
-					if ( G[ij] == 1 ) psi[ij] = rnorm( 0, 1 );
+					if ( G[ij] == 1 ) 
+						psi[ij] = rnorm( 0, 1 );
+					else
+						psi[ij] = 0.0;
 				}
 			
 			for( i = 0; i < dim - 1; i++ ) 
@@ -333,7 +366,11 @@ void log_exp_mc( int G[], int nu[], int *b, double H[], int *check_H, int *mc, i
 						//psi[i, j] = - sum( psi[ i, i : ( j - 1 ) ] * H[ i : ( j - 1 ), j ] )
 						sumPsiH = 0.0;
 						for( h = i; h < j; h++ )
+						{
+							if( sumPsiH == R_PosInf ) sumPsiH = max_numeric_limits_ld;	
+							if( sumPsiH == R_NegInf ) sumPsiH = min_numeric_limits_ld;	
 							sumPsiH += ( psi[ h * dim + i ] * H[ j * dim + h ] ); 
+						}
 						psi[ij] = - sumPsiH;
 						
 						if ( i > 0 )  //if ( i > 1 )
@@ -342,7 +379,11 @@ void log_exp_mc( int G[], int nu[], int *b, double H[], int *check_H, int *mc, i
 								//sum( psi[ r, r : i ] * H[ r : i, i ] )
 								sumPsiHi = 0.0;
 								for( h = r; h < i + 1; h++  )
-									sumPsiHi += ( psi[ h * dim + r ] * H[ i * dim + h ] );
+								{
+									if( sumPsiHi == R_PosInf ) sumPsiHi = max_numeric_limits_ld;	
+									if( sumPsiHi == R_NegInf ) sumPsiHi = min_numeric_limits_ld;	
+									sumPsiHi += ( psi[ h * dim + r ] * H[ i * dim + h ] );	
+								}
 									
 								//sum( psi[ r, r : j ] * H[ r : j, j ] ) )
 								sumPsiHj = 0.0;
@@ -353,10 +394,16 @@ void log_exp_mc( int G[], int nu[], int *b, double H[], int *check_H, int *mc, i
 								psi[ij] -= ( sumPsiHi * sumPsiHj ) / psi[i * dim + i];
 							}
 
+						if( psi[ij] == R_PosInf ) psi[ij] = max_numeric_limits_ld;	
+						if( psi[ij] == R_NegInf ) psi[ij] = min_numeric_limits_ld;	
+
 						//f_T[k] <- f_T[k] + psi[i, j] ^ 2
 						f_T[iter] += ( psi[ij] * psi[ij] ); 
 					}
 				}
+		
+			// checking Inf values
+			if( f_T[iter] == R_PosInf ) f_T[iter] = max_numeric_limits_ld;			
 		}
 	}
 	PutRNGstate();	
